UNIT WECom;
{$I WEGLOBAL.PAS}
{ -- Asychronous communications routines for WWIVEdit 2.4  --
         Completely re-worked by Wayne Hoxsie 5-27-94
}
INTERFACE

PROCEDURE AsyncTransmit(ch1:char);
FUNCTION  AsyncRecieve:char;
PROCEDURE AsyncFlushComm;
FUNCTION  AsyncCharWaiting:BOOLEAN;
FUNCTION  AsyncCDetect:boolean;
PROCEDURE AsyncInit(port_num:byte);
PROCEDURE AsyncShutdown;

IMPLEMENTATION

USES DOS;

CONST
  port_base_id : array[0..4] of integer = (0,$3f8,$2f8,$2e8,$3e8);
  port_IRQ : array[0..4] of byte = (0,4,3,4,3);
  IER=1;
  FCR=2;
  IIR=2;
  LCR=3;
  MCR=4;
  LSR=5;
  MSR=6;

VAR
  buf_head,buf_tail : integer;
  com_buf : array[0..1024] of byte;
  flow_control : boolean;
  async_irq,port_base : byte;

PROCEDURE Disable; INLINE($FA);
PROCEDURE Enable;  INLINE($FB);

PROCEDURE async_isr; INTERRUPT;
BEGIN
  com_buf[buf_head]:=port[port_base];
  inc(buf_head);
  if buf_head = 1024 then
    buf_head := 0;
  port[$20]:=$20;
END;

PROCEDURE AsyncTransmit(ch1:char);
VAR
  temp : byte;   

BEGIN
  repeat
    temp:=port[port_base + LSR] AND $20;
  until temp=0;
  if flow_control then
  repeat
    temp :=port[port_base + MSR] AND $10;
  until temp=0;
  port[port_base]:=integer(ch1);
END;

FUNCTION  AsyncCharWaiting:BOOLEAN;
BEGIN
  asynccharwaiting := buf_head<>buf_tail
END;

FUNCTION  AsyncRecieve:char;
VAR
  c1 : char;
BEGIN
  if buf_head <> buf_tail then
  BEGIN
    disable;
    c1 := char(com_buf[buf_tail]);
    inc(buf_tail);
    if buf_tail = 1024 then
      buf_tail := 0;
    enable;
    asyncrecieve:=c1;
  END  else
    asyncrecieve:=char(0);
END;

PROCEDURE AsyncFlushComm;
BEGIN
  disable;
  buf_head :=0;
  buf_tail :=0;
  enable;
END;

PROCEDURE dtr(i : integer);
VAR
  i1 : integer;
BEGIN
  i1 := port[port_base + MCR] AND $00FE;
  if i<>0 then
  port[port_base + MCR]:= i1+1 else
  port[port_base + MCR] := i1;
END;

FUNCTION  AsyncCDetect:boolean;
BEGIN
  asynccdetect := (port[port_base + MSR] AND $0080)<> 0;
END;

PROCEDURE AsyncInit(port_num : byte);
VAR
  temp : integer;
BEGIN
  flow_control:=FALSE;
  port_base := port_base_id[port_num];
  async_irq := port_IRQ[port_num];
  setintvec(8 + async_irq, addr(async_isr));
  buf_head :=0;
  buf_tail := 0;
  port[port_base + LCR]:= $03; { set N81 parameters }
  disable;
  temp := port[port_base + LSR];   { Read LSR to reset all bits }
  temp := port[port_base];         { clear any character still pending }
  temp := port[$21];               { read the PIC mask)
  temp := temp AND ((1 SHL async_irq) XOR $00FF);  {set appropriate bit }
  port[$21]:=temp;                 {set the PIC mask appropriately}
  port[port_base + FCR]:=$c1;      {set FIFO operation }
  temp:=port[port_base + MCR];
  port[port_base + MCR]:= temp OR $0A; {Set MCR }
  {port[port_base + IIR]:=$c0;}
  enable;
  dtr(1);
END;

PROCEDURE AsyncShutdown;
VAR
  temp : integer;
  oldvec : pointer;
BEGIN
  if port_base<>0 then BEGIN
    disable;
    temp := port[$21];
    temp := temp OR ((1 SHL async_irq));
    port[$21]:= temp;
    port[port_base + IIR]:=0;
    port[port_base + MCR]:=3;
    getintvec(8,oldvec);
    setintvec(async_irq+8,oldvec);
    enable;
    port_base:=0;
  END;
END;

END.